// 140mm fan duct 45-60°
// Mikhail Svarichevsky
// https://3.14.by
use <path_extrude.scad>;

external_size = 140;
hole_spacing = 125;
corner_radius = 5;

module deflector(max_angle=50, a1 = 180, a2 = 360) {
    myPoints = [
        // first expression generating the points in the positive Y quadrant
        [ 42*cos(a1)+27*(((a1<90) || (a1>270))?1:-1),42*sin(a1)+8.8],
        for(t = [a1+5:5:a2-5]) [ 42*cos(t)+27*(((t<90) || (t>270))?1:-1),42*sin(t)],
        [ 42*cos(a2)+27*(((a2<90) || (a2>270))?1:-1),42*sin(a2)+8.8],
        // second expression generating the points in the negative Y quadrant
        [ 40*cos(a2)+27*(((a2<90) || (a2>270))?1:-1),40*sin(a2)+8.8],
        for(t = [a2-5:-5:a1+5]) [ 40*cos(t)+27*(((t<90) || (t>270))?1:-1),40*sin(t)],
        [ 40*cos(a1)+27*(((a1<90) || (a1>270))?1:-1),40*sin(a1)+8.8]
    ];

    
    myPath = [10*[0,cos(0),sin(0)], 10*[0,cos(1),sin(1)], for(t = [2:2:max_angle]) 10*[0,cos(t),sin(t)] ];
    //myPath = [10*[0,cos(0),sin(0)], 10*[0,cos(0.1),sin(0.1)], 10*[0,cos(20),sin(20)], 10*[0,cos(50),sin(50)], 10*[0,cos(70),sin(70)]];
    path_extrude(exPath=myPath, exShape=myPoints, merge=false);
}

    union(){
        difference()
        {
            //main mounting plate
            roundedcube([external_size, external_size, 4], center = true, radius = corner_radius, apply_to = "xy");
            union()
            {
                //central hole
                translate([0, 0, -3]){roundedcube([external_size-6, external_size-6, 10], center = true, radius = 40, apply_to = "xy");};
                //4 mounting points
                translate([-hole_spacing/2, -hole_spacing/2, -3]){cylinder(r=2.5,h=10);};
                translate([+hole_spacing/2, -hole_spacing/2, -3]){cylinder(r=2.5,h=10);};
                translate([-hole_spacing/2, +hole_spacing/2, -3]){cylinder(r=2.5,h=10);};
                translate([+hole_spacing/2, +hole_spacing/2, -3]){cylinder(r=2.5,h=10);};
            }
        };
        
        
        difference()
        {
            union()
            {
                union()
                {   
                    translate([0, -8+25, 3])deflector(45);
                    translate([0, -8+25-60, 3])deflector(55);
                }
                union()
                {   
                    translate([0, -8+25-30, 3])deflector(50);
                    translate([0, -8+25-90, 3])deflector(60);
                }
            };
            union()
            {
                //for (dx = [-25 : 25: 25])
                //{
                    //translate([dx,0,50])
                        //rotate([0,45,0])
                            //cube([20,140,20], center = true);
                //}
                //Teeth
                /*sp = 27;
                for (dx = [-sp*1.5 : sp: sp*1.5])
                {
                    translate([dx,0,0])
                        rotate([0,45,0])
                            cube([15,100,15], center = true);
                }
                for (dx = [-sp : sp: sp])
                {
                    translate([dx,0,-4])
                        rotate([0,45,0])
                            cube([15,100,15], center = true);
                }*/
                translate([0,-158,94])
                    cube([200,200,200], center = true);
                
            };
        };
};


//union(){o1;o2};


module roundedcube(size, center, radius, apply_to = "all") {
    
    x = size[0];
    y = size[1];
    linear_extrude(height=size[2])
    hull()
    {
        // place 4 circles in the corners, with the given radius
        translate([(-x/2)+(radius), (-y/2)+(radius), 0])
        circle($fn = 40, r=radius);
    
        translate([(x/2)-(radius), (-y/2)+(radius), 0])
        circle($fn = 40, r=radius);
    
        translate([(-x/2)+(radius), (y/2)-(radius), 0])
        circle($fn = 40, r=radius);
    
        translate([(x/2)-(radius), (y/2)-(radius), 0])
        circle($fn = 40, r=radius);
    }
}